from .algorithms import Algorithms, PemAlgorithms, PlainAlgorithms
from .claims import JWTDecodeClaims, JWTEncodeClaims
from .decoder import JWTDecoder
from .encoder import JWTEncoder
from .exceptions import JWTDecodeException, JWTEncodeException, JWTException
from .identities import Identity, PemIdentity, PlainIdentity

__all__ = (
    "Algorithms",
    "PemAlgorithms",
    "PlainAlgorithms",
    "JWTDecodeException",
    "JWTEncodeException",
    "JWTException",
    "Identity",
    "PemIdentity",
    "PlainIdentity",
    "JWTEncoder",
    "JWTDecoder",
    "JWTEncodeClaims",
    "JWTDecodeClaims",
)



import abc
from dataclasses import dataclass
from typing import Any, Callable

from .algorithms import Algorithms, PemAlgorithms, PlainAlgorithms


class Identity(abc.ABC):
    algorithm: Algorithms

    @abc.abstractmethod
    def get_encode_key(self) -> Any:
        raise NotImplementedError

    @abc.abstractmethod
    def get_decode_key(self) -> Any:
        raise NotImplementedError


@dataclass
class PlainIdentity(Identity):
    secret_key_loader: Callable[[], str]
    algorithm: PlainAlgorithms

    secret_key: str | None = None

    def get_encode_key(self) -> Any:
        if self.secret_key is None:
            return self.load_secret_key()
        return self.secret_key

    def load_secret_key(self) -> str:
        self.secret_key = self.secret_key_loader()
        return self.secret_key

    def get_decode_key(self) -> Any:
        if self.secret_key is None:
            return self.load_secret_key()
        return self.secret_key


@dataclass
class PemIdentity(Identity):
    algorithm: PemAlgorithms
    public_key_loader: Callable[[], bytes] | None = None
    private_key_loader: Callable[[], bytes] | None = None

    public_key: bytes | None = None
    private_key: bytes | None = None

    def get_encode_key(self) -> Any:
        if self.private_key is None:
            return self.load_private_key()
        return self.private_key

    def load_private_key(self) -> bytes:
        if self.private_key_loader is None:
            raise ValueError("Private key loader is not defined")
        self.private_key = self.private_key_loader()
        return self.private_key

    def get_decode_key(self) -> Any:
        if self.public_key is None:
            return self.load_public_key()
        return self.public_key

    def load_public_key(self) -> bytes:
        if self.public_key_loader is None:
            raise ValueError("Public key loader is not defined")
        self.public_key = self.public_key_loader()
        return self.public_key



import logging
from datetime import datetime, timezone
from typing import Any

from jwt import PyJWT, PyJWTError, get_unverified_header

from .claims import JWTEncodeClaims
from .exceptions import JWTEncodeException
from .identities import Identity


class JWTEncoder:
    identity: Identity
    jwt: PyJWT
    logger: logging.Logger
    default_claims: JWTEncodeClaims

    def __init__(
        self,
        identity: Identity,
        claims: JWTEncodeClaims,
        py_jwt: PyJWT = PyJWT(),  # type: ignore
        logger: logging.Logger = logging.getLogger(__name__),
    ):
        self.jwt = py_jwt
        self.logger = logger
        self.identity = identity
        self.default_claims = claims

    def encode(
        self, data: dict[str, Any], claims: JWTEncodeClaims | None = None
    ) -> str:
        timestamp = datetime.now(tz=timezone.utc)
        if claims is None:
            claims = self.default_claims
        return self._encode(
            payload=self.create_payload(data, claims, timestamp),
            key=self.identity.get_encode_key(),
            algorithm=self.identity.algorithm.value,
        )

    def create_payload(
        self,
        data: dict[str, Any],
        claims: JWTEncodeClaims,
        timestamp: datetime,
    ) -> dict[str, Any]:
        payload = claims.get_defined(timestamp)
        payload["payload"] = data
        return payload

    def _encode(self, **options: Any) -> str:
        try:
            self.logger.debug(f'Encoding "{options}".')
            return self.jwt.encode(**options)
        except PyJWTError as error:
            self.logger.debug(
                f'Interrupt encoding "{options}". Error: {error}'
            )
            raise JWTEncodeException(error) from error

    def get_headers(self, token: str) -> dict[str, str]:
        return get_unverified_header(token)

    def get_identity(self) -> Identity:
        return self.identity



from dataclasses import dataclass, field
from datetime import datetime, timedelta
from typing import Any

from .utils import to_unix_timestamp


@dataclass(frozen=True)
class JWTEncodeClaims:
    set_iat: bool = True
    iss: str | None = None
    aud: list[str] | None = None
    exp: timedelta | None = None
    nbf: timedelta | None = None

    def get_defined(self, timestamp: datetime) -> dict[str, Any]:
        claims: dict[str, Any] = {}
        if self.exp:
            claims["exp"] = to_unix_timestamp(timestamp + self.exp)
        if self.nbf:
            claims["nbf"] = to_unix_timestamp(timestamp + self.nbf)
        if self.iss:
            claims["iss"] = self.iss
        if self.aud:
            claims["aud"] = self.aud
        if self.set_iat:
            claims["iat"] = to_unix_timestamp(timestamp)
        return claims


@dataclass
class JWTDecodeClaims:
    issuer: str | None = None
    leeway: int | None = None
    audience: str | None = None
    require: list[str] = field(default_factory=lambda: ["exp", "iat", "nbf"])

    def get_defined(self) -> dict[str, Any]:
        claims: dict[str, Any] = {"require": self.require}
        if self.issuer:
            claims["issuer"] = self.issuer
        if self.leeway:
            claims["leeway"] = self.leeway
        if self.audience:
            claims["audience"] = self.audience
        return claims



import logging
from typing import Any

from jwt import PyJWT, PyJWTError, get_unverified_header

from .claims import JWTDecodeClaims
from .exceptions import JWTDecodeException
from .identities import Identity


class JWTDecoder:
    identity: Identity
    jwt: PyJWT
    logger: logging.Logger
    default_claims: JWTDecodeClaims

    def __init__(
        self,
        identity: Identity,
        claims: JWTDecodeClaims,
        py_jwt: PyJWT = PyJWT(),  # type: ignore
        logger: logging.Logger = logging.getLogger(__name__),
    ):
        self.jwt = py_jwt
        self.logger = logger
        self.identity = identity
        self.default_claims = claims

    def decode(
        self, token: str, claims: JWTDecodeClaims | None = None
    ) -> dict[str, Any]:
        if claims is None:
            claims = self.default_claims
        return self._decode(
            jwt=token,
            key=self.identity.get_decode_key(),
            algorithms=[self.identity.algorithm.value],
            options=claims.get_defined(),
        )

    def _decode(self, **options: Any) -> dict[str, Any]:
        try:
            self.logger.debug(f'Decoding "{options}".')
            return self.jwt.decode(**options)
        except PyJWTError as error:
            self.logger.debug(
                f'Interrupt decoding "{options}". Error: {error}'
            )
            raise JWTDecodeException(error) from error

    def get_headers(self, token: str) -> dict[str, str]:
        return get_unverified_header(token)

    def get_identity(self) -> Identity:
        return self.identity



from enum import Enum


class Algorithms(Enum):
    pass


class PlainAlgorithms(Algorithms):
    HS256 = "HS256"
    HS384 = "HS384"
    HS512 = "HS512"


class PemAlgorithms(Algorithms):
    ES256 = "ES256"
    ES256K = "ES256K"
    ES384 = "ES384"
    ES512 = "ES512"
    RS256 = "RS256"
    RS384 = "RS384"
    RS512 = "RS512"
    PS256 = "PS256"
    PS384 = "PS384"
    PS512 = "PS512"
    EdDSA = "EdDSA"



import os
from pathlib import Path

from setuptools import setup  # type: ignore [import]


setup(
    name="jwts",
    version=os.environ["GITHUB_REF_NAME"],
    description="Set of abstractions for jwt tokens based on pyjwt[crypto].",
    author="Vladimir Vojtenko",
    author_email="vladimirdev635@gmail.com",
    license="MIT",
    packages=["jwts"],
    install_requires=["pyjwt[crypto]"],
    package_data={"jwts": ["py.typed"]},
    long_description=(Path(__file__).parent / "README.md").read_text(),
    long_description_content_type="text/markdown",
)
